@@ -1,7 +1,7 @@
 #include <limits.h>
 
-#include "Mutils.h"
 #include <R_ext/Lapack.h>
+#include "Mutils.h"
 
 // La_norm_type() & La_rcond_type()  have been in R_ext/Lapack.h
 //  but have still not been available to package writers ...
@@ -43,33 +43,38 @@ char La_rcond_type(const char *typstr)
 
 double get_double_by_name(SEXP obj, char *nm)
 {
-    SEXP nms = getAttrib(obj, R_NamesSymbol);
+    SEXP nms = PROTECT(getAttrib(obj, R_NamesSymbol));
     int i, len = length(obj);
 
     if ((!isReal(obj)) || (length(obj) > 0 && nms == R_NilValue))
 	error(_("object must be a named, numeric vector"));
     for (i = 0; i < len; i++) {
 	if (!strcmp(nm, CHAR(STRING_ELT(nms, i)))) {
+	    UNPROTECT(1);
 	    return REAL(obj)[i];
 	}
     }
+    UNPROTECT(1);
     return R_NaReal;
 }
 
 SEXP
 set_double_by_name(SEXP obj, double val, char *nm)
 {
-    SEXP nms = getAttrib(obj, R_NamesSymbol);
+    SEXP nms = PROTECT(getAttrib(obj, R_NamesSymbol));
     int i, len = length(obj);
 
     if ((!isReal(obj)) || (length(obj) > 0 && nms == R_NilValue))
 	error(_("object must be a named, numeric vector"));
+    // case 1:  replace existing entry named  <nm>
     for (i = 0; i < len; i++) {
 	if (!strcmp(nm, CHAR(STRING_ELT(nms, i)))) {
 	    REAL(obj)[i] = val;
+	    UNPROTECT(1);
 	    return obj;
 	}
     }
+    // case 2:  no such name --> add new entry with that name at end of vec
     {
 	SEXP nx = PROTECT(allocVector(REALSXP, len + 1)),
 	    nnms = allocVector(STRSXP, len + 1);
@@ -81,7 +86,7 @@ set_double_by_name(SEXP obj, double val, char *nm)
 	}
 	REAL(nx)[len] = val;
 	SET_STRING_ELT(nnms, len, mkChar(nm));
-	UNPROTECT(1);
+	UNPROTECT(2);
 	return nx;
     }
 }
@@ -129,18 +134,18 @@ SEXP get_factors(SEXP obj, char *nm)
 */
 SEXP set_factors(SEXP obj, SEXP val, char *nm)
 {
+    PROTECT(val); /* set_factors(..) may be called as "finalizer" after UNPROTECT()*/
     SEXP fac = GET_SLOT(obj, Matrix_factorSym),
-	nms = getAttrib(fac, R_NamesSymbol);
+	nms = PROTECT(getAttrib(fac, R_NamesSymbol));
     int i, len = length(fac);
 
     if ((!isNewList(fac)) || (length(fac) > 0 && nms == R_NilValue))
 	error(_("'factors' slot must be a named list"));
-    PROTECT(val); /* set_factors(..) may be called as "finalizer" after UNPROTECT()*/
     // if there's already a 'nm' factor, we replace it and return:
     for (i = 0; i < len; i++) {
 	if (!strcmp(nm, CHAR(STRING_ELT(nms, i)))) {
 	    SET_VECTOR_ELT(fac, i, duplicate(val));
-	    UNPROTECT(1);
+	    UNPROTECT(2);
 	    return val;
 	}
     }
@@ -158,7 +163,7 @@ SEXP set_factors(SEXP obj, SEXP val, char *nm)
     SET_VECTOR_ELT(nfac, len, duplicate(val));
     SET_STRING_ELT(nnms, len, mkChar(nm));
     SET_SLOT(obj, Matrix_factorSym, nfac);
-    UNPROTECT(3);
+    UNPROTECT(4);
     return VECTOR_ELT(nfac, len);
 }
 
@@ -174,6 +179,27 @@ SEXP R_set_factors(SEXP obj, SEXP val, SEXP name, SEXP warn)
     }
 }
 
+// R interface for emptying the '@ factors' slot of *function *argument* 'obj' [CARE!]
+SEXP R_empty_factors(SEXP obj, SEXP warn)
+{
+    Rboolean do_warn = asLogical(warn), modified = FALSE;
+    PROTECT(obj);
+    if(R_has_slot(obj, Matrix_factorSym)) {
+	SEXP fac = GET_SLOT(obj, Matrix_factorSym);
+	if (length(fac) > 0) { // if there's a non-empty factor slot, replace it with list()
+	    SEXP fac = PROTECT(allocVector(VECSXP, 0));
+	    SET_SLOT(obj, Matrix_factorSym, fac);
+	    modified = TRUE;
+	    UNPROTECT(1);
+	}
+    } else {
+	if(do_warn) warning(_("Matrix object has no 'factors' slot"));
+    }
+    UNPROTECT(1);
+    return ScalarLogical(modified);
+}
+
+
 #if 0 				/* unused */
 /* useful for all the ..CMatrix classes (and ..R by [0] <-> [1]); but unused */
 SEXP CMatrix_set_Dim(SEXP x, int nrow)
@@ -339,16 +365,28 @@ SEXP Dim_validate(SEXP obj, SEXP name) {
 			CHAR(STRING_ELT(name, 0)));
 }
 
-// obj must have @Dim and @Dimnames. Assume 'Dim' is already checked.
-SEXP dimNames_validate(SEXP obj)
+/**
+ * Validate dimnames (the @Dim slot typically), assuming
+ * that 'dims' is already checked.
+ *
+ * Utility, called from  dimNames_validate(), but available to other routines.
+ *
+ * @param dmNms an R object should be valid `dimnames`
+ * @param dims  an integer vector of length 2 (must have been checked by caller!).
+
+ * @return a SEXP, either TRUE (= success) or an error message string ("character")
+ */
+SEXP dimNames_validate__(SEXP dmNms, int dims[], const char* obj_name)
 {
-    int *dims = INTEGER(GET_SLOT(obj, Matrix_DimSym));
-    SEXP dmNms = GET_SLOT(obj, Matrix_DimNamesSym);
-    if(!isNewList(dmNms))
-	return mkString(_("Dimnames slot is not a list"));
-    if(length(dmNms) != 2)
-	return mkString(_("Dimnames slot is a list, but not of length 2"));
     char buf[99];
+    if(!isNewList(dmNms)) {
+	sprintf(buf, _("%s is not a list"), obj_name);
+	return mkString(buf);
+    }
+    if(length(dmNms) != 2) {
+	sprintf(buf, _("%s is a list, but not of length 2"), obj_name);
+	return mkString(buf);
+    }
     for(int j=0; j < 2; j++) { // x@Dimnames[j] must be NULL or character(length(x@Dim[j]))
 	if(!isNull(VECTOR_ELT(dmNms, j))) {
 	    if(TYPEOF(VECTOR_ELT(dmNms, j)) != STRSXP) {
@@ -367,27 +405,46 @@ SEXP dimNames_validate(SEXP obj)
 }
 
 
+/**
+ * Check R (Matrix-like) object: must have @Dim and @Dimnames.
+ * Assume 'Dim' is already checked.
+ * (typically used in  S4 validity method)
+ *
+ * @param obj an R object (typically inheriting from `Matrix`)
+ *
+ * @return a SEXP, either TRUE (= success) or an error message string ("character")
+ */
+SEXP dimNames_validate(SEXP obj)
+{
+    return dimNames_validate__(/* dmNms = */ GET_SLOT(obj, Matrix_DimNamesSym),
+			       /* dims =  */ INTEGER(GET_SLOT(obj, Matrix_DimSym)),
+			       "Dimnames slot");
+}
 
-#define PACKED_TO_FULL(TYPE)						\
-TYPE *packed_to_full_ ## TYPE(TYPE *dest, const TYPE *src,		\
-		        int n, enum CBLAS_UPLO uplo)			\
-{									\
-    int i, j, pos = 0;							\
-									\
-    AZERO(dest, n*n);							\
-    for (j = 0; j < n; j++) {						\
-	switch(uplo) {							\
-	case UPP:							\
-	    for (i = 0; i <= j; i++) dest[i + j * n] = src[pos++];	\
-	    break;							\
-	case LOW:							\
-	    for (i = j; i < n; i++) dest[i + j * n] = src[pos++];	\
-	    break;							\
-	default:							\
-	    error(_("'uplo' must be UPP or LOW"));			\
-	}								\
-    }									\
-    return dest;							\
+
+
+#define PACKED_TO_FULL(TYPE)					\
+TYPE *packed_to_full_ ## TYPE(TYPE *dest, const TYPE *src,	\
+		        int n, enum CBLAS_UPLO uplo)		\
+{								\
+    int i, j, pos = 0;						\
+    size_t n2 = n * (size_t)n;					\
+								\
+    AZERO(dest, n2);						\
+    for (j = 0; j < n; j++) {					\
+        size_t jn = j * (size_t)n;				\
+	switch(uplo) {						\
+	case UPP:						\
+	    for (i = 0; i <= j; i++) dest[i + jn] = src[pos++];	\
+	    break;						\
+	case LOW:						\
+	    for (i = j; i < n; i++)  dest[i + jn] = src[pos++];	\
+	    break;						\
+	default:						\
+	    error(_("'uplo' must be UPP or LOW"));		\
+	}							\
+    }								\
+    return dest;						\
 }
 
 PACKED_TO_FULL(double)
@@ -458,6 +515,8 @@ void l_packed_getDiag(int *dest, SEXP x, int n)
 
 //----------------------------------------------------------------------
 
+/** diag(x) <- D  for   x a  <dspMatrix>  or dppMatrix, ..etc
+ */
 SEXP d_packed_setDiag(double *diag, int l_d, SEXP x, int n)
 {
 #define SET_packed_setDiag				\
@@ -497,7 +556,9 @@ SEXP l_packed_setDiag(int *diag, int l_d, SEXP x, int n)
 #define tr_END_packed_setDiag						\
     if (*diag_P(x) == 'U') { /* uni-triangular */			\
 	/* after setting, typically is not uni-triangular anymore: */	\
-	SET_STRING_ELT(GET_SLOT(ret, Matrix_diagSym), 0, mkChar("N"));	\
+	SEXP ch_N = PROTECT(mkChar("N"));				\
+	SET_STRING_ELT(GET_SLOT(ret, Matrix_diagSym), 0, ch_N);		\
+	UNPROTECT(1);							\
     }									\
     END_packed_setDiag
 
@@ -539,8 +600,11 @@ SEXP d_packed_addDiag(double *diag, int l_d, SEXP x, int n)
 SEXP tr_d_packed_addDiag(double *diag, int l_d, SEXP x, int n)
 {
     SEXP ret = PROTECT(d_packed_addDiag(diag, l_d, x, n));
-    if (*diag_P(x) == 'U') /* uni-triangular */
-	SET_STRING_ELT(GET_SLOT(ret, Matrix_diagSym), 0, mkChar("N"));
+    if (*diag_P(x) == 'U') { /* uni-triangular */
+	SEXP ch_N = PROTECT(mkChar("N"));
+	SET_STRING_ELT(GET_SLOT(ret, Matrix_diagSym), 0, ch_N);
+	UNPROTECT(1);
+    }
     UNPROTECT(1);
     return ret;
 }
@@ -613,10 +677,11 @@ install_diagonal(double *dest, SEXP A)
     int nc = INTEGER(GET_SLOT(A, Matrix_DimSym))[0];
     int i, ncp1 = nc + 1, unit = *diag_P(A) == 'U';
     double *ax = REAL(GET_SLOT(A, Matrix_xSym));
+    size_t in1 = 0;
 
-    AZERO(dest, nc * nc);
-    for (i = 0; i < nc; i++)
-	dest[i * ncp1] = (unit) ? 1. : ax[i];
+    AZERO(dest, nc * (size_t)nc);
+    for (i = 0; i < nc; i++, in1 += ncp1) // in1 == i * ncp1
+	dest[in1] = (unit) ? 1. : ax[i];
     return dest;
 }
 
@@ -626,10 +691,11 @@ install_diagonal_int(int *dest, SEXP A)
     int nc = INTEGER(GET_SLOT(A, Matrix_DimSym))[0];
     int i, ncp1 = nc + 1, unit = *diag_P(A) == 'U';
     int *ax = INTEGER(GET_SLOT(A, Matrix_xSym));
+    size_t in1 = 0;
 
-    AZERO(dest, nc * nc);
-    for (i = 0; i < nc; i++)
-	dest[i * ncp1] = (unit) ? 1 : ax[i];
+    AZERO(dest, nc * (size_t)nc);
+    for (i = 0; i < nc; i++, in1 += ncp1) // in1 == i * ncp1
+	dest[in1] = (unit) ? 1 : ax[i];
     return dest;
 }
 
@@ -657,7 +723,7 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 	MATRIX_VALID_ndense, /* 5  */
 	""};
     SEXP ans, ad = R_NilValue, an = R_NilValue;	/* -Wall */
-    int sz, ctype = R_check_class_etc(A, valid),
+    int ctype = R_check_class_etc(A, valid),
 	nprot = 1;
     enum dense_enum M_type = ddense /* -Wall */;
 
@@ -697,10 +763,9 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 		dd[0] = LENGTH(A);					\
 		dd[1] = 1;						\
 	    }								\
-	    SEXP nms = getAttrib(A, R_NamesSymbol);			\
+	    SEXP nms = PROTECT(getAttrib(A, R_NamesSymbol)); nprot++;	\
 	    if(nms != R_NilValue) {					\
-		an = PROTECT(allocVector(VECSXP, 2));			\
-		nprot++;						\
+		an = PROTECT(allocVector(VECSXP, 2)); nprot++;		\
 	        SET_VECTOR_ELT(an, (transpose_if_vec)? 1 : 0, nms);	\
 		/* not needed: SET_VECTOR_ELT(an, 1, R_NilValue); */    \
 	    } /* else nms = NULL ==> an remains NULL */                 \
@@ -710,16 +775,17 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 	DUP_MMATRIX_NON_CLASS(FALSE);
     }
 
-    ans = PROTECT(NEW_OBJECT(MAKE_CLASS(M_type == ddense ? "dgeMatrix" :
-					(M_type == ldense ? "lgeMatrix" :
-					 "ngeMatrix"))));
-#define DUP_MMATRIX_SET_1					\
-    SET_SLOT(ans, Matrix_DimSym, duplicate(ad));		\
-    SET_SLOT(ans, Matrix_DimNamesSym, (LENGTH(an) == 2) ? 	\
-	     duplicate(an): allocVector(VECSXP, 2));		\
-    sz = INTEGER(ad)[0] * INTEGER(ad)[1]
+    ans = PROTECT(NEW_OBJECT_OF_CLASS(M_type == ddense ? "dgeMatrix" :
+				      (M_type == ldense ? "lgeMatrix" :
+					 "ngeMatrix")));
+#define DUP_MMATRIX_SET_1(_IS_SYM_) do {				\
+    SET_SLOT(ans, Matrix_DimSym, duplicate(ad));			\
+    SET_SLOT(ans, Matrix_DimNamesSym, _IS_SYM_ ? symmetric_DimNames(an) : \
+       (!isNull(an) && LENGTH(an) == 2) ? duplicate(an): allocVector(VECSXP,2)); \
+} while(0)
 
-    DUP_MMATRIX_SET_1;
+    R_xlen_t sz = ((R_xlen_t) INTEGER(ad)[0]) * INTEGER(ad)[1];
+    Rboolean is_symm = FALSE;
 
     if(M_type == ddense) { /* ddense -> dge */
 
@@ -744,6 +810,7 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 	case 14:	 		/* ---	corMatrix */			\
 	    Memcpy(ansx, REAL(GET_SLOT(A, Matrix_xSym)), sz);			\
 	    make_d_matrix_symmetric(ansx, A);					\
+	    is_symm = TRUE;							\
 	    break;								\
 	case 5:			/* ddiMatrix */					\
 	    install_diagonal(ansx, A);						\
@@ -761,10 +828,12 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 				  INTEGER(ad)[0],				\
 				  *uplo_P(A) == 'U' ? UPP : LOW);		\
 	    make_d_matrix_symmetric(ansx, A);					\
+	    is_symm = TRUE;							\
 	    break;								\
 	}  /* switch(ctype) */
 
-	DUP_MMATRIX_ddense_CASES;
+	DUP_MMATRIX_ddense_CASES; // also setting  is_symm
+	DUP_MMATRIX_SET_1(is_symm);
     }
     else { /* M_type == ldense || M_type = ndense  */
 	/* ldense -> lge */
@@ -789,6 +858,7 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 	case 3+14+6:			/* nsyMatrix */
 	    Memcpy(ansx, LOGICAL(GET_SLOT(A, Matrix_xSym)), sz);
 	    make_i_matrix_symmetric(ansx, A);
+	    is_symm = TRUE;
 	    break;
 	case 4+14:			/* ldiMatrix */
 	    // case 4+14+6:      /* ndiMatrix _DOES NOT EXIST_ */
@@ -807,12 +877,15 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 			       INTEGER(ad)[0],
 			       *uplo_P(A) == 'U' ? UPP : LOW);
 	    make_i_matrix_symmetric(ansx, A);
+	    is_symm = TRUE;
 	    break;
 
 	default:
 	    error(_("unexpected ctype = %d in dup_mMatrix_as_geMatrix"), ctype);
 	}  /* switch(ctype) */
 
+	DUP_MMATRIX_SET_1(is_symm);
+
     }  /* if(M_type == .) */
 
     UNPROTECT(nprot);
@@ -821,11 +894,10 @@ SEXP dup_mMatrix_as_geMatrix(SEXP A)
 
 SEXP dup_mMatrix_as_dgeMatrix2(SEXP A, Rboolean tr_if_vec)
 {
-    SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))),
+    SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),
 	ad = R_NilValue , an = R_NilValue;	/* -Wall */
     static const char *valid[] = {"_NOT_A_CLASS_", MATRIX_VALID_ddense, ""};
-    int ctype = R_check_class_etc(A, valid),
-	nprot = 1, sz;
+    int ctype = R_check_class_etc(A, valid), nprot = 1;
     double *ansx;
 
     if (ctype > 0) {		/* a ddenseMatrix object */
@@ -835,8 +907,7 @@ SEXP dup_mMatrix_as_dgeMatrix2(SEXP A, Rboolean tr_if_vec)
     else if (ctype < 0) {	/* not a (recognized) classed matrix */
 	if (!isReal(A)) {
 	    if (isInteger(A) || isLogical(A)) {
-		A = PROTECT(coerceVector(A, REALSXP));
-		nprot++;
+		A = PROTECT(coerceVector(A, REALSXP)); nprot++;
 	    } else
 		error(_("invalid class '%s' to dup_mMatrix_as_dgeMatrix"),
 		      class_P(A));
@@ -844,8 +915,10 @@ SEXP dup_mMatrix_as_dgeMatrix2(SEXP A, Rboolean tr_if_vec)
 	DUP_MMATRIX_NON_CLASS(tr_if_vec);
     }
 
-    DUP_MMATRIX_SET_1;
-    DUP_MMATRIX_ddense_CASES;
+    R_xlen_t sz = ((R_xlen_t) INTEGER(ad)[0]) * INTEGER(ad)[1];
+    Rboolean is_symm = FALSE;
+    DUP_MMATRIX_ddense_CASES; // also setting  is_symm
+    DUP_MMATRIX_SET_1(is_symm);
     UNPROTECT(nprot);
     return ans;
 }
@@ -856,14 +929,14 @@ SEXP dup_mMatrix_as_dgeMatrix(SEXP A) {
 
 SEXP new_dgeMatrix(int nrow, int ncol)
 {
-    SEXP ans = PROTECT(NEW_OBJECT(MAKE_CLASS("dgeMatrix"))),
+    SEXP ans = PROTECT(NEW_OBJECT_OF_CLASS("dgeMatrix")),
 	 ad = PROTECT(allocVector(INTSXP, 2));
 
     INTEGER(ad)[0] = nrow;
     INTEGER(ad)[1] = ncol;
     SET_SLOT(ans, Matrix_DimSym, ad);
     SET_SLOT(ans, Matrix_DimNamesSym, allocVector(VECSXP, 2));
-    ALLOC_SLOT(ans, Matrix_xSym, REALSXP, nrow * ncol);
+    ALLOC_SLOT(ans, Matrix_xSym, REALSXP, ((R_xlen_t) nrow) * ncol);
 
     UNPROTECT(2);
     return ans;
@@ -1122,26 +1195,26 @@ SEXP Mmatrix(SEXP args)
     int *d_a = INTEGER(GET_SLOT(a, Matrix_DimSym)),
 	*d_b = INTEGER(GET_SLOT(b, Matrix_DimSym)),
 	n1 = d_a[0], m = d_a[1],
-	n2 = d_b[0],
-	nprot = 1;
-    SEXP ans,
-	a_x = GET_SLOT(a, Matrix_xSym),
-	b_x = GET_SLOT(b, Matrix_xSym);
+	n2 = d_b[0];
     if(d_b[1] != m)
 	error(_("the number of columns differ in R_rbind2_vector: %d != %d"),
 	      m, d_b[1]);
+    SEXP
+	a_x = GET_SLOT(a, Matrix_xSym),
+	b_x = GET_SLOT(b, Matrix_xSym);
+    int nprot = 1;
     // Care: can have "ddenseMatrix" "ldenseMatrix" or "ndenseMatrix"
     if(TYPEOF(a_x) != TYPEOF(b_x)) { // choose the "common type"
 	// Now know: either LGLSXP or REALSXP. FIXME for iMatrix, zMatrix,..
-	if(TYPEOF(a_x) != REALSXP)
-	    a_x = PROTECT(duplicate(coerceVector(a_x, REALSXP)));
-	else if(TYPEOF(b_x) != REALSXP)
-	    b_x = PROTECT(duplicate(coerceVector(b_x, REALSXP)));
-	nprot++;
+	if(TYPEOF(a_x) != REALSXP) {
+	    a_x = PROTECT(duplicate(coerceVector(a_x, REALSXP))); nprot++;
+	} else if(TYPEOF(b_x) != REALSXP) {
+	    b_x = PROTECT(duplicate(coerceVector(b_x, REALSXP))); nprot++;
+	}
     }
 
-    ans = PROTECT(allocVector(TYPEOF(a_x), m * (n1 + n2)));
-    int i, ii = 0;
+    SEXP ans = PROTECT(allocVector(TYPEOF(a_x), m * (n1 + n2)));
+    int ii = 0;
     switch(TYPEOF(a_x)) {
     case LGLSXP: {
 	int
@@ -1149,12 +1222,11 @@ SEXP Mmatrix(SEXP args)
 	    *ax= LOGICAL(a_x),
 	    *bx= LOGICAL(b_x);
 
-#define COPY_a_AND_b_j						\
-/*  FIXME faster: use Memcpy() : */				\
-	for(int j=0; j < m; j++) {				\
-	    for(i=0; i < n1; i++) r[ii++] = ax[j*n1 + i];	\
-	    for(i=0; i < n2; i++) r[ii++] = bx[j*n2 + i];	\
-	}
+#define COPY_a_AND_b_j					\
+	for(int j=0; j < m; j++) {			\
+	    Memcpy(r+ii, ax+ j*n1, n1); ii += n1;	\
+	    Memcpy(r+ii, bx+ j*n2, n2); ii += n2;	\
+	} ; break
 
 	COPY_a_AND_b_j;
     }
@@ -1167,7 +1239,6 @@ SEXP Mmatrix(SEXP args)
 	COPY_a_AND_b_j;
     }
     } // switch
-
     UNPROTECT(nprot);
     return ans;
 }
@@ -1283,15 +1354,14 @@ SEXP symmetric_DimNames(SEXP dn) {
 	if (isNull(VECTOR_ELT(dn,1)))					\
 	    SET_VECTOR_ELT(dn, 1, VECTOR_ELT(dn, 0));			\
 	if(do_nm) { /* names(dimnames(.)): */				\
-	    SEXP nms_dn = getAttrib(dn, R_NamesSymbol);			\
+	    SEXP nms_dn = PROTECT(getAttrib(dn, R_NamesSymbol));	\
 	    if(!R_compute_identical(STRING_ELT(nms_dn, 0),		\
 				    STRING_ELT(nms_dn, 1), 16)) {	\
-		PROTECT(nms_dn);					\
 		int J = LENGTH(STRING_ELT(nms_dn, 0)) == 0; /* 0/1 */   \
 		SET_STRING_ELT(nms_dn, !J, STRING_ELT(nms_dn, J));	\
 		setAttrib(dn, R_NamesSymbol, nms_dn);			\
-		UNPROTECT(1);						\
             }								\
+	    UNPROTECT(1);						\
 	}								\
 	UNPROTECT(1)
 
@@ -1317,7 +1387,6 @@ SEXP R_symmetric_Dimnames(SEXP x) {
     return symmetric_DimNames(GET_SLOT(x, Matrix_DimNamesSym));
 }
 
-
 /**
  * Set 'Dimnames' slot of 'dest' from the one of 'src' when
  * 'src' is a "symmetricMatrix" with possibly asymmetric dimnames,
@@ -1337,3 +1406,15 @@ void SET_DimNames_symm(SEXP dest, SEXP src) {
     return;
 }
 
+/**
+ * A safe NEW_OBJECT(MAKE_CLASS(cls)),  where the caller must protect the
+ * return value of this function
+ *
+ * @param an R character string specifying the name of a known S4 class
+ */
+SEXP NEW_OBJECT_OF_CLASS(const char* cls)
+{
+    SEXP ans = NEW_OBJECT(PROTECT(MAKE_CLASS(cls)));
+    UNPROTECT(1);
+    return ans;
+}
