@@ -248,6 +248,9 @@ static void chTr2Ralloc(CHM_TR dest, CHM_TR src)
  * @return ans containing pointers to the slots of x, *unless*
  *	check_Udiag and x is unitriangular.
  */
+/* AS_CHM_SP  (x) := as_cholmod_sparse((CHM_SP)alloca(sizeof(cholmod_sparse)), x, TRUE,  FALSE)
+ * AS_CHM_SP__(x) := as_cholmod_sparse((CHM_SP)alloca(sizeof(cholmod_sparse)), x, FALSE, FALSE)
+ */
 CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 			 Rboolean check_Udiag, Rboolean sort_in_place)
 {
@@ -276,7 +279,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
      * utility, but the lengths of x and i should agree. */
     ans->nzmax = LENGTH(islot);
 				/* values depending on ctype */
-    ans->x = xpt(ctype, x);
+    ans->x     = xpt  (ctype, x);
     ans->stype = stype(ctype, x);
     ans->xtype = xtype(ctype);
 
@@ -303,7 +306,7 @@ CHM_SP as_cholmod_sparse(CHM_SP ans, SEXP x,
 	}
     }
 
-    if (check_Udiag && ctype % 3 == 2 // triangular
+    if (check_Udiag && ctype % 3 == 2 /* triangular */ && ans->nrow // fails for Dim = (0,0)
 	&& (*diag_P(x) == 'U')) { /* diagU2N(.)  "in place" : */
 	double one[] = {1, 0};
 	CHM_SP eye = cholmod_speye(ans->nrow, ans->ncol, ans->xtype, &c);
@@ -825,7 +828,10 @@ int R_cholmod_start(CHM_CM c)
     int res;
     if (!(res = cholmod_start(c)))
 	error(_("Unable to initialize cholmod: error code %d"), res);
-    c->print_function = R_cholmod_printf; /* Rprintf gives warning */
+    /*SuiteSparse <= 4.x.y :
+     * c->print_function = R_cholmod_printf; /. Rprintf gives warning */
+    SuiteSparse_config.printf_func = R_cholmod_printf;/* Rprintf gives warning */
+    //                                 ^^^^^^^^^ now is misnomer
     /* Since we provide an error handler, it may not be a good idea to allow CHOLMOD printing,
      * because that's not easily suppressed on the R level :
      * Hence consider, at least temporarily *  c->print_function = NULL; */
